%

%
% Copyright (c) 2015 LAAS/CNRS
% All rights reserved.
%
% Redistribution  and  use  in  source  and binary  forms,  with  or  without
% modification, are permitted provided that the following conditions are met:
%
%   1. Redistributions of  source  code must retain the  above copyright
%      notice and this list of conditions.
%   2. Redistributions in binary form must reproduce the above copyright
%      notice and  this list of  conditions in the  documentation and/or
%      other materials provided with the distribution.
%
% THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
% WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
% MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
% ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
% WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
% ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
% IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
%
%                                           Anthony Mallet on Thu Jun 25 2015
%
classdef client < handle
  properties (Access = private)
    id
  end

  methods
    function self = client(dev, path, speed)
      % tk3.client - Create connections to a tk3-mikrokopter device.
      %
      % c = tk3.client('path', speed)
      %    Connect to the serial port 'path' at 'speed' baud rate
      %
      % tk3.client Methods:
      %    rpath - Add a new load path on the server.
      %    load - Load a component on the server.
      %    update - Process received events.
      %
      self.id = tk3_comm('connect', dev, path, speed);
    end

    function delete(self)
      % delete - Disconnect an instance of a tk3.client.
      %
      tk3_comm('disconnect', self.id);
    end

    function start(self)
      tk3_comm('start', self.id);
    end
    function stop(self)
      tk3_comm('stop', self.id);
    end
    function velocity(self, vels)
      if length(vels) == 1
        tk3_comm('vel', self.id, vels);
      else
        tk3_comm('vels', self.id, vels);
      end
    end

    function get_imu(self, frequency)
      tk3_comm('imu', self.id, frequency);
    end
    function get_motor(self, frequency)
      tk3_comm('motor', self.id, frequency);
    end
    function get_battery(self, frequency)
      tk3_comm('battery', self.id, frequency);
    end
    function d = data(self)
      d = tk3_comm('data', self.id);
    end

    function gyro_calibrate(self, duration)
      tk3_comm('gcalib', self.id, duration);
    end
  end
end
