/*
 * Copyright (c) 2015 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 *                                      Anthony Mallet on Mon Feb 16 2015
 */
#ifndef H_LIBTK3
#define H_LIBTK3

#include <sys/types.h>
#include <sys/stat.h>

#include <pthread.h>
#include <stdbool.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>

enum tk3_device {
  TK3_NONE,
  TK3_MKBL,
  TK3_MKFL
};

struct tk3_data_stamp {
  uint8_t pseq;
  double pts;
  double offset;
};

struct tk3_mkbl_state {
  bool fresh;

  uint8_t id;
  bool emerg, servo, spinning, starting;

  double velocity, pwm, current;
  double m_ts;
  struct tk3_data_stamp m_stamp;
};

struct tk3_mkfl_state {
  struct { bool imu, motor, battery; } fresh;

  double battery;
  double battery_ts;
  struct tk3_data_stamp battery_stamp;

  double acc_x, acc_y, acc_z;
  double gyr_r, gyr_p, gyr_y;
  double imu_ts;
  struct tk3_data_stamp imu_stamp;

  struct tk3_mkbl_state *motor;
  int nmotor;
};

struct tk3_data {
  enum tk3_device dev;
  union {
    struct tk3_mkbl_state mkbl;
    struct tk3_mkfl_state mkfl;
  };

  uint32_t battery_p, imu_p, motor_p;
};

struct tk3_channel_s {
  char path[1024];
  int fd, nrefs;
  dev_t st_dev;
  ino_t st_ino;

  uint8_t buf[64], r, w; /* read ring buffer */

  bool start;
  bool escape;
  uint8_t msg[64], len; /* last message */

  struct tk3_data data;
};


int	tk3_client(enum tk3_device dev, const char *path, uint32_t speed,
                char *emsg, size_t emsgsize);
struct tk3_data
	tk3_data(int channel, char *emsg, size_t emsgsize);
int	tk3_close(int channel, char *emsg, size_t emsgsize);
int	tk3_send_data(int channel, const char *data, double *vals, size_t len,
                char *emsg, size_t emsgsize);
int	tk3_rqst_data(int channel, const char *data, double frequency,
                char *emsg, size_t emsgsize);
int	tk3_gyro_calib(int channel, int duration, char *emsg, size_t emsgsize);


int	tk3_open_tty(const char *device, uint32_t baud);
int	tk3_wait_msg(struct tk3_channel_s **channels, int n,
                pthread_mutex_t *mutex);
int	tk3_recv_msg(struct tk3_channel_s *chan, bool block);
int	tk3_send_msg(const struct tk3_channel_s *chan, const char *fmt, ...);

#endif /* H_LIBTK3 */
